/** cube3.cpp
*
*   Demonstrate OpenGL functions
*
*      Add animation
*
**/
#define WIN32_LEAN_AND_MEAN
#include <stdlib.h>
#include <math.h>
#include <windows.h>
#include "dib.h"
#include "GLapplic.h"
#include "tb.h"

#define OBJ_CUBE  1

class MyApp: public CAppWindow
{
	public:
		MyApp(char *name);
		int nAngle;
		void SetupWindow();
		void CleanupWindow();
		void OnSize(int cx, int cy);
		void WmTimer();
		void WmLButtonDown(int keys, POINT pt) { tbButtonDown(pt.x,pt.y); }
		void WmMouseMove(int keys, POINT pt) 
		{ 
			tbMotion(pt.x,pt.y); 	
			InvalidateRect (hWnd, NULL, FALSE);         
		}
		void WmLButtonUp(int keys, POINT pt) { tbButtonUp(); }
		void PrepareScene();
		void DrawScene();
};

MyApp::MyApp(char *name): CAppWindow(name)
{
	Width  = 480 ;
	Height = 360 ;
	nAngle = 70;
}

void MyApp::SetupWindow()
{
	SetTimer (hWnd, 1, 50, NULL);
	tbInit();
}

void MyApp::CleanupWindow()
{
	KillTimer(hWnd, 1);
}

void MyApp::OnSize(int cx, int cy)
{
	CAppWindow::OnSize(cx,cy);
	tbReshape(cx,cy);
}

void MyApp::WmTimer()
{
	//tbStepAnimation();
	InvalidateRect (hWnd, NULL, FALSE);         
}

void InitializeRC (void)
{
	GLfloat glfLightAmbient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
	GLfloat glfLightDiffuse[] = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLfloat glfLightSpecular[] = { 0.0f, 0.0f, 0.0f, 1.0f };

	//
	// Enable depth testing and backface culling.
	//
	glEnable (GL_DEPTH_TEST);
	glEnable (GL_CULL_FACE);

	//
	// Add a light to the scene.
	//
	glLightfv (GL_LIGHT0, GL_AMBIENT, glfLightAmbient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, glfLightDiffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, glfLightSpecular);
	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);
}

CDib dib[6];
unsigned char *pixel;

void makeCheckerboard()
{
	int nrows, ncols;
	nrows = 64;
	ncols = 64;
	pixel = new unsigned char[3*nrows*ncols];
	int count = 0;
	int c;
	for (int i=0; i<nrows; i++) {
		for (int j=0; j<ncols; j++) {
			c = (((i/8)+(j/8))%2) * 255;
			pixel[count++] = c;
			pixel[count++] = c;
			pixel[count++] = 0;
		}
	}
}

void setTexture(CDib &dib, int texture_id)
{
	int width, height;
	glBindTexture(GL_TEXTURE_2D,texture_id);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	width = dib.GetWidth();
	height = dib.GetHeight();
	int nbits = dib.GetBitsPerPixel();
	unsigned char *offset = dib.GetBits();
	int count = 0;
	// swap position of red and blue bytes (RGB vs BGR)
	for (int j=0; j<height; j++) {
		for (int i=0; i<width; i++) {
			unsigned char c = offset[count];
			offset[count] = offset[count+2];
			offset[count+2] = c;
			count += 3;
		}
	}

	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,
		width,height,0,GL_RGB,GL_UNSIGNED_BYTE,dib.GetBits());
	//width = 64;
	//height = 64;
	//glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,width,height,
	//	0,GL_RGB,GL_UNSIGNED_BYTE,pixel);
}

void MakeCube()
{
	glNewList(OBJ_CUBE, GL_COMPILE) ;

	//
	// Draw the six faces of the cube.
	//
	// Front face
	makeCheckerboard();
	dib[0].Read("front.bmp");
	glEnable(GL_TEXTURE_2D);
	setTexture(dib[0],2000);
	glBegin (GL_POLYGON);
	glNormal3f (0.0f, 0.0f, 1.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f (1.0f, 1.0f, 1.0f);
	glTexCoord2f(0.0f, 1.0f);
	glVertex3f (-1.0f, 1.0f, 1.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f (-1.0f, -1.0f, 1.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex3f (1.0f, -1.0f, 1.0f);            
	glEnd ();

	// Back face
	dib[2].Read("back.bmp");
	glEnable(GL_TEXTURE_2D);
	setTexture(dib[2],2002);
	glBegin (GL_POLYGON);
	glNormal3f (0.0f, 0.0f, -1.0f);
	glTexCoord2f(0.0f, 1.0f);
	glVertex3f (1.0f, 1.0f, -1.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f (1.0f, -1.0f, -1.0f);           
	glTexCoord2f(1.0f, 0.0f);
	glVertex3f (-1.0f, -1.0f, -1.0f);          
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f (-1.0f, 1.0f, -1.0f);           
	glEnd ();

	// Right face
	dib[3].Read("right.bmp");
	setTexture(dib[3],2003);
	glBegin (GL_POLYGON);
	glNormal3f (-1.0f, 0.0f, 0.0f);
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f (-1.0f, 1.0f, 1.0f);
	glTexCoord2f(0.0f, 1.0f);
	glVertex3f (-1.0f, 1.0f, -1.0f);           
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f (-1.0f, -1.0f, -1.0f);          
	glTexCoord2f(1.0f, 0.0f);
	glVertex3f (-1.0f, -1.0f, 1.0f);           
	glEnd ();

	// Left face
	dib[1].Read("left.bmp");
	glEnable(GL_TEXTURE_2D);
	setTexture(dib[1],2001);
	glBegin (GL_POLYGON);
	glNormal3f (1.0f, 0.0f, 0.0f);
	glTexCoord2f(0.0f, 1.0f);
	glVertex3f (1.0f, 1.0f, 1.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f (1.0f, -1.0f, 1.0f);            
	glTexCoord2f(1.0f, 0.0f);
	glVertex3f (1.0f, -1.0f, -1.0f);           
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f (1.0f, 1.0f, -1.0f);            
	glEnd ();

	// Top face
	dib[4].Read("top.bmp");
	setTexture(dib[4],2004);
	glBegin (GL_POLYGON);
	glNormal3f (0.0f, 1.0f, 0.0f);
	glTexCoord2f(0.0f, 1.0f);
	glVertex3f (-1.0f, 1.0f, -1.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f (-1.0f, 1.0f, 1.0f);            
	glTexCoord2f(1.0f, 0.0f);
	glVertex3f (1.0f, 1.0f, 1.0f);             
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f (1.0f, 1.0f, -1.0f);            
	glEnd ();

	// Bottom face
	dib[5].Read("bottom.bmp");
	setTexture(dib[5],2005);
	glBegin (GL_POLYGON);
	glNormal3f (0.0f, -1.0f, 0.0f);
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f (-1.0f, -1.0f, -1.0f);
	glTexCoord2f(1.0f, 0.0f);
	glVertex3f (1.0f, -1.0f, -1.0f);           
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f (1.0f, -1.0f, 1.0f);            
	glTexCoord2f(0.0f, 1.0f);
	glVertex3f (-1.0f, -1.0f, 1.0f);           
	glEnd ();
	glDisable(GL_TEXTURE_2D);

	glEndList() ;			   
}

void MyApp::PrepareScene()
{
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	InitializeRC();
	MakeCube();
}



void MyApp::DrawScene()
{
	GLfloat glfLightAmbient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
	GLfloat glfLightDiffuse[] = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLfloat glfLightSpecular[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	GLfloat glfMaterialColor[] = { 0.0f, 0.0f, 1.0f, 1.0f };

	//
	// Enable depth testing and clear the color and depth buffers.
	//
	glEnable (GL_DEPTH_TEST);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//
	// Add a light to the scene.
	//
	glLightfv (GL_LIGHT0, GL_AMBIENT, glfLightAmbient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, glfLightDiffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, glfLightSpecular);
	glEnable (GL_LIGHTING);
	glEnable (GL_LIGHT0);

	//
	// Define the modelview transformation.
	//
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	glTranslatef (0.0f, 0.0f, -8.0f);
	//glRotatef (30.0f, 1.0f, 0.0f, 0.0f);
	//glRotatef ((GLfloat) nAngle, 0.0f, 1.0f, 0.0f);
	tbMatrix();

	//
	// Define the reflective properties of the cube's faces.
	//
	glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, glfMaterialColor);


	// Draw the cube
	glCallList(OBJ_CUBE) ;

	//
	// Flush the drawing pipeline.
	//
	glFlush ();
}

int Main()
{
	return MyApp("trackball").Run();
}
